//3D Printed Arduino Servo Platform With Robotic Arm Code

//Platform configurations
/*  Motor1  Forward   Motor2
    Left              Right
    Motor3  Backward  Motor4
*/

#include <Wire.h>
#include <PCA9685.h>
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>

//Intitializtion & Declaration & Definition  
RF24 radio (8, 9); //CE, CSN
const byte address[6] = "00002";

typedef struct { //declare stucture, This is the structure the platform will receive from controller
  int Lxj; //members
  int Lyj;
  int Rxj;
  int Ryj;
  int RJBState;
}Data;
Data value; //value is object

int i = 1; // for selecting servo joints or angle value to change
int Joint[6] = {-9999, 5, 90, 90, 90, 90}; //Servo Joint angle values
int j = 1; // for indicating connection, like if controller is connected or not
int k = 1; // for selecting which servo on the arm to control

int ackData[5] = { -1, 1, 1, 0, 90}; //ackData array includes data that will be send back to the controller

unsigned long pT = 0; //Previous Time
const long interval = 500; //After half a second the controller turn off the Tank will Stop

//Setting up the PCA9685 servo driver
PCA9685 driver;
int Motor1, Motor2, Motor3, Motor4, J0, diff;
// PCA9685 outputs = 12-bit = 4096 steps
// 0.7ms to 2.3ms is the PWM signal for FS90R Servo 
// First calculate pulse width or period 1/50Hz = 0.02 second = 20ms (ms = milliseconds); 
// 0.7ms/20ms = 0.035 (3.5% of 20ms is lowest); 
// 2.3ms/20ms = 0.115  (11.5% of 20ms is the highest);  
// 4096*0.035 = 144 steps ; 4096*0.115 = 471 steps 
PCA9685_ServoEval pwmServo(144, 307, 471); // (-90deg, +90deg)

// 0.5ms to 2.5ms is the PWM signal for MG90S servo motor () 
// 2.5% of 20ms = 0.5ms ; 12.5% of 20ms = 2.5ms
// 2.5% of 4096 = 102 steps; 12.5% of 4096 = 512 steps
PCA9685_ServoEval pwmServoS(102, 512); // (-90deg, +90deg)


void setup() {
  Serial.begin(9600);

  Wire.begin();               // Wire must be started first
  Wire.setClock(400000);      // Supported baud rates are 100kHz, 400kHz, and 1000kHz
  driver.resetDevices();      // Software resets all PCA9685 devices on Wire line
  driver.init();              // Address pins A5-A0 set to B000000
  driver.setPWMFrequency(50); // Set frequency to 50Hz
  driver.setChannelOff(0);    //initialized all servo motors
  driver.setChannelOff(1);
  driver.setChannelOff(2);
  driver.setChannelOff(3);
  
  driver.setChannelPWM(10, pwmServoS.pwmForAngle(0));
  delay(2000);
  driver.setChannelPWM(11, pwmServoS.pwmForAngle(90));  //Left
  delay(2000);
  driver.setChannelPWM(12, pwmServoS.pwmForAngle(-90)); //Right
  delay(2000);
  driver.setChannelPWM(13, pwmServoS.pwmForAngle(90));
  delay(2000);
  driver.setChannelPWM(14, pwmServoS.pwmForAngle(90));
  delay(2000);
  driver.setChannelPWM(15, pwmServoS.pwmForAngle(90));
  delay(2000);
  Motor1 = 0;
  Motor2 = 0;
  Motor3 = 0;
  Motor4 = 0;
  
  radio.begin();
  radio.setDataRate( RF24_250KBPS );
  radio.enableAckPayload();
  radio.openReadingPipe(1, address);
  radio.setPALevel(RF24_PA_MIN);
  radio.startListening();
  Serial.print("Start");
  
}  

void loop() {
  unsigned long cT = millis(); //Current Time
  
  //Receive Data from controller 
  if ( radio.available() ){
    radio.read( &value, sizeof(value) );

    radio.writeAckPayload(1, &ackData, sizeof(ackData)); //Sending data back to the controller 
    j = 0;

    pT = cT; //make previous time equal current time so it recounts

    //Forward
    if (value.Lxj > 540 && (value.Lyj > 480) && (value.Lyj < 540)) {
      Motor1 = map(value.Lxj, 540, 1023, 0, 90);    
      Motor2 = map(value.Lxj, 540, 1023, 0, -90);   
      Motor3 = Motor1;
      Motor4 = Motor2;
      ackData[0] = 3;
    }
    //Backward
    else if (value.Lxj < 480 && (value.Lyj > 480) && (value.Lyj < 540)) {
      Motor1 = map(value.Lxj, 0, 480, -90, 0);
      Motor2 = map(value.Lxj, 0, 480, 90, 0);
      Motor3 = Motor1;
      Motor4 = Motor2;
      ackData[0] = 4;
    }
    //Rotate conuterclockwise 
    else if ((value.Lyj < 480) && (value.Lxj > 480) && (value.Lxj < 540)) {
      Motor1 = map(value.Lyj, 480, 0, -90, 0);
      Motor2 = map(value.Lyj, 480, 0, 0, -90);  
      Motor3 = Motor1;
      Motor4 = Motor2;
      ackData[0] = 6;
    }
    //Rotate clockwise 
    else if ((value.Lyj > 540) && (value.Lxj > 480) && (value.Lxj < 540)) {
      Motor1 = map(value.Lyj, 540, 1023, 0, 90);
      Motor2 = map(value.Lyj, 540, 1023, 90, 0);
      Motor3 = Motor1;
      Motor4 = Motor2;
      ackData[0] = 5;
    }

    else {  //When vehicle not moving
      Motor1 = 0;
      Motor2 = 0;
      Motor3 = 0;
      Motor4 = 0;
      ackData[0] = 2;
    }

    if (ackData[0] == 2) { //If controller is not connected turn off the FS90R servo motors
      driver.setChannelOff(0);
      driver.setChannelOff(1);
      driver.setChannelOff(2);
      driver.setChannelOff(3);
    }

    //Actuating the FS90R motors
    if (Motor1 == 0){ 
      driver.setChannelOff(0);
    } 
    else {
      driver.setChannelPWM(0, pwmServo.pwmForAngle(Motor1));
    }
    if (Motor2 == 0){
      driver.setChannelOff(1);
    } 
    else {
      driver.setChannelPWM(1, pwmServo.pwmForAngle(Motor2));
    }
    if (Motor3 == 0){
      driver.setChannelOff(2);
    }
    else {
      driver.setChannelPWM(2, pwmServo.pwmForAngle(Motor3));
    } 
    if (Motor4 == 0){
      driver.setChannelOff(3);
    }
    else {
      driver.setChannelPWM(3, pwmServo.pwmForAngle(Motor4));
    } 

    // Right analog stick for changing servo angle or selecting MG90S servo motors (arm)
    if (value.RJBState == LOW) {
      i ++;
      delay(100);
      if (i > 2) {
        i = 1;
      } 
      Serial.println("Button Pressed!");   //If the switch is pressed, print message
    }

    //For increasing or decreasing MG90S servo angles
    if (j == 0 && i == 2) {
      if (value.Ryj > 750){
        Joint[k] ++;
        if (Joint[k] > 90){
          Joint[k] = 90;
        }
      } 
      if (value.Ryj < 250) {
        Joint[k] --;
        if (Joint[k] < -90){
          Joint[k] = -90;
        }
      }
    }

    //For selecting which MG90S servos to change
    if (j == 0 && i == 1) {
      if (value.Rxj > 750) {        
        k ++;
        delay(50);
        if (k > 5){
          k = 1;
        }
      }
      if (value.Rxj < 250) {
        k --;
        delay(50);
        if (k < 1){
          k = 5;
        }
      }
    }   

    //Actuating the MG90S servos 
    driver.setChannelPWM(10, pwmServoS.pwmForAngle(Joint[1]));    //Base Servo motor
    driver.setChannelPWM(11, pwmServoS.pwmForAngle(Joint[2]));    //Link 1 Servo motor 1
    driver.setChannelPWM(12, pwmServoS.pwmForAngle(-Joint[2]));   //Link 1 Servo motor 2
    driver.setChannelPWM(13, pwmServoS.pwmForAngle(Joint[3]));    //Link 2 Servo motor    
    driver.setChannelPWM(14, pwmServoS.pwmForAngle(Joint[4]));    //Wrist rotation Servo motor
    driver.setChannelPWM(15, pwmServoS.pwmForAngle(Joint[5]));    //Gripper Servo motor

    //Update the ackData array
    ackData[1] = i;
    ackData[2] = j;
    ackData[3] = k;
    ackData[4] = Joint[k];

    //Print values for troubleshooting
    Serial.print(" Check ");
    Serial.print("  Lyj: ");
    Serial.print(value.Lyj);
    Serial.print("  Lxj: ");
    Serial.print(value.Lxj);
    Serial.print("  Joint[1]: ");
    Serial.print(Joint[1]);
    Serial.print("  size ");
    Serial.print(sizeof(ackData));
    Serial.print("  ackData0 ");
    Serial.print(ackData[0]);
    Serial.print("  cT: ");
    Serial.print(cT);
    Serial.print("  pT: ");
    Serial.print(pT);
    Serial.print("  RJB: ");
    Serial.print(value.RJBState);
    Serial.print("  Joint[k]: ");
    Serial.println(Joint[k]);

    
  } else { // When platform not getting data from controller it stops the platform
    if (cT - pT >= interval) {
      pT = cT;
      i = 0;
      j = 1;
      Serial.println(" No Signal ");
      Serial.print(" OFF ");
      Serial.print("  Lyj: ");
      Serial.print(value.Lyj);
      Serial.print("  Lxj: ");
      Serial.print(value.Lxj);
      Serial.print("  Motor1: ");
      Serial.print(Motor1);
      Serial.print("  Motor2: ");
      Serial.println(Motor2);
      Motor1 = 0;
      Motor2 = 0;
      Motor3 = 0;
      Motor4 = 0;
      driver.setChannelOff(0);
      driver.setChannelOff(1);
      driver.setChannelOff(2);
      driver.setChannelOff(3);
      delay(500);
    }
  }
}
